package org.okiproject.serviceapi.file;
/*
Copyright (c) 2001 Massachusetts Institute of Technology

Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice
shall be included in all copies or substantial portions of
the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/file/Cabinet.java,v $
 */
/**
Cabinets have properties, and contain other Cabinets and ByteStores.
<p>
They may manage quotas, that is, if the implementation supports
quotas, each Agent may be assigned a quota of space used in the
Cabinet.
<p>
Cabinets contain CabinetEntries, each of which may be a ByteStore or a
Cabinet.  They are known by their GUIDs and optionally by name, where
the name is a string with no syntax, and may represent a filename,
e.g. logo.gif or a name and path within a heirarchy,
e.g. images/logos/logo1.gif
<p>
It is a goal that a Cabinet be representable as a jar file and vice
versa.  A Cabinet may be converted to a jar file using a method.
<p>
One special type of Cabinet is used to store URLs, the implementation
is built around java.net.URL.  The name of the ByteStore is a URL.  The
cabinet is read-only, doesn't support quotas or versioning.
<p>
ByteStores and Cabinets are added to Cabinets.  Cabinets are created
in CabinetFactories and ByteStores are created in Cabinets.
<hr>
@version $Revision: 1.8 $ / $Date: 2002/02/16 05:02:53 $
*/
public interface Cabinet
extends java.io.Serializable {

  /**
   * Method getProperties
   *
   * return properties of this cabinet
   *
   * @return java.util.Map
   *
   */
  public java.util.Map getProperties();

  /**
   * Method createTempByteStore
   *
   * create new ByteStore
   *
   * @return created ByteStore
   *
   */
  public ByteStore createTempByteStore();

  /**
   * Method createByteStore
   *
   * anonymous, known only by GUID
   *
   * @return ByteStore
   *
   */
  public ByteStore createByteStore();


  /**
   * Method createByteStore
   *
   *
   * named, error if name already exists
   *
   * @param name
   *
   * @return ByteStore
   *
   */
  public ByteStore createByteStore(String name);

  /**
   * Method createByteStore
   *
   *
   * create new anonymous ByteStore in this Cabinet by copying contents and
   * appropriate attributes of another ByteStore.
   *
   * @param old
   *
   * @return ByteStore
   *
   */
  public ByteStore createByteStore(ByteStore old);

  /**
   * Method createByteStore
   *
   *
   * create new named ByteStore in this Cabinet by copying contents and
   * appropriate attributes of another ByteStore.
   *
   * @param name
   * @param old
   *
   * @return ByteStore
   *
   */
  public ByteStore createByteStore(String name, ByteStore old);


  /**
   * Method add
   *
   * add subcabinet, must be from same CabinetFactory
   *
   *
   * @param subCabinet
   *
   */
  public void add(Cabinet subCabinet);


  /**
   * Method add
   *
   * add ByteStore, must be from same CabinetFactory
   *
   * @param byteStore
   *
   */
  public void add(ByteStore byteStore);


  /**
   * Method remove
   *
   * remove ByteStore or Cabinet.  Does not destroy Cabinet or ByteStore
   *
   *
   * @param subCabinet
   *
   */
  public void remove(Cabinet subCabinet);


  /**
   * Method remove
   *
   *
   * @param byteStore
   *
   */
  public void remove(ByteStore byteStore);



  /**
   * Method setQuota
   *
   * set quota for Agent in this Cabinet
   *
   * @param owner
   * @param quotaBytes
   *
   */
  public void setQuota(org.okiproject.serviceapi.common.Agent owner,
                       long quotaBytes);

  /**
   * Method getQuota
   *
   *
   * @param owner
   *
   * @return long
   *
   */
  public long getQuota(org.okiproject.serviceapi.common.Agent owner);

  /**
   * Method getQuotaUsed
   *
   *
   * @param owner
   *
   * @return long
   *
   */
  public long getQuotaUsed(org.okiproject.serviceapi.common.Agent owner);


  /**
   * Method getAvailableBytes
   *
   *
   * @return long, space available in Cabinet, in bytes
   *
   */
  public long getAvailableBytes();

  /**
   * Method getUsedBytes
   *
   *
   * @return long, space used in Cabinet, in bytes
   *
   */
  public long getUsedBytes();


  /**
   * Method getByteStore
   *
   * get ByteStore from Cabinet by guid.
   *
   * @param guid
   *
   * @return ByteStore
   *
   */
  public ByteStore getByteStore(org.okiproject.serviceapi.guid.GUID guid);



  /**
   * Method getByteStore
   *
   * get by name.  Not all ByteStores have names,
   * but if it has a name, it is unique within a Cabinet 
   *
   * @param name
   *
   * @return ByteStore
   *
   */
  public ByteStore getByteStore(String name);


  // These are analogous to jar:

  /**
   * Method entries
   *
   * analogous to jar:
   *
   * @return java.util.Enumeration
   *
   */
  public java.util.Enumeration entries();

  /**
   * Method getCabinetEntry
   *
   * analogous to jar:
   *
   * @param guid
   *
   * @return CabinetEntry which has given GUID
   *
   */
  public CabinetEntry getCabinetEntry(
    org.okiproject.serviceapi.guid.GUID guid);

  /**
   * Method getCabinetEntry
   *
   *
   * @param name
   *
   * @return CabinetEntry which has given name
   *
   */
  public CabinetEntry getCabinetEntry(String name);


  /**
   * Method returnAsJar
   *
   * produce jar file of cabinet and its contents
   *
   * @return java.util.jar.JarFile containing this cabinet and all its
   * contents
   *
   */
  java.util.jar.JarFile returnAsJar();
}

