package org.okiproject.serviceapi.file;
/*
Copyright (c) 2001 Massachusetts Institute of Technology

Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice
shall be included in all copies or substantial portions of
the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/file/CabinetFactory.java,v $
 */
/**
 <p> An installation may have several implementations of CabinetFactory
 available.
<p>
The application calls either
<code>org.okiproject.serviceapi.file.getDefaultCabinetFactory(agent)</code>
or
<code>org.okiproject.serviceapi.file.getAvailableCabinetFactories()</code>
and after examining the features of the available CabinentFacory
implementations, calls
<code>org.okiproject.serviceapi.file.getCabinetFactory(agent, properties)</code>
to get a CabinetFactory to use.
<p>
Once there is a CabinetFactory implementation available, a new Cabinet
is created from the factory using <code>createCabinet()</code>, e.g.
<code>cabinetFactoryImpl.createCabinet()</code>
<p>
Or, to reuse an existing cabinet, use
<code>org.okiproject.serviceapi.file.getCabinet(guid)</code>
<p>
A CabinetFactory implementation has some of the characteristics of a
filesystem implementation.  Different filesystems support different
maximum file lengths, some may be journaled, some may support RAID.
<hr>
@version $Revision: 1.9 $ / $Date: 2002/02/16 05:02:53 $
*/
public class CabinetFactory
implements java.io.Serializable {
  /**
   * Method getDefaultCabinetFactory
   *
   * get system default cabinet factory
   *
   *
   * @param agent
   * @return CabinetFactory
   *
   */
  public static CabinetFactory getDefaultCabinetFactory(
          org.okiproject.serviceapi.common.Agent agent)
  throws Exception {
    java.util.Map properties = getPropertiesFromPRB();

    return getCabinetFactory(agent, properties);
  }

  protected static java.util.Properties getPropertiesFromPRB()
    throws Exception {
    java.util.PropertyResourceBundle prb =
      (java.util.PropertyResourceBundle) java.util.ResourceBundle.getBundle(
        "CabinetFactory");

    if (null != prb) {
      java.util.Properties  properties = new java.util.Properties();
      java.util.Enumeration keys       = prb.getKeys();

      while (keys.hasMoreElements()) {
        String key = (String) keys.nextElement();

        properties.setProperty(key, (String) prb.handleGetObject(key));
      }

      return properties;
    }
    throw new org.okiproject.serviceapi.file.CabinetException(
       org.okiproject.serviceapi.file.CabinetException.CABINETFACTORYPROPERTIESDIDNOTLOAD);
  }

  /**
   * Method getAvailableCabinetFactories
   *
   * return types of CabinetFactory implementations available in the
   * installation
   *
   *
   * @param agent
   * @return java.util.List containing all factories available
   *
   */
  public static java.util.List getAvailableCabinetFactories(
          org.okiproject.serviceapi.common.Agent agent)
  throws Exception {
    throw new java.lang.UnsupportedOperationException();
  }

  /**
   * Method getCabinetFactory
   *
   *
   * @param agent
   * @param properties
   *
   * @return CabinetFactory
   *
   * @throws Exception
   *
   */
  public static CabinetFactory getCabinetFactory(
          org.okiproject.serviceapi.common.Agent agent,
          java.util.Map properties)
  throws Exception {
    throw new java.lang.UnsupportedOperationException();
  }

  private org.okiproject.serviceapi.common.Agent agent = null;

  /**
   * Method getAgent
   *
   *
   * @return agent for which this cabinet was created
   *
   */
  public org.okiproject.serviceapi.common.Agent getAgent() {
    return this.agent;
  }

  /**
   * Method setAgent
   *
   *
   * @param agent
   *
   */
  public void setAgent(org.okiproject.serviceapi.common.Agent agent) {
    this.agent = agent;
  }

  private java.util.Map properties = null;

  /**
   * Method getProperties
   *
   * capability properties include:
   * replication (yes/no)
   * versioning (yes/no)
   * exclusive control to OKI system (yes/no)
   * supports quota (yes/no)
   * readonly/readwrite
   * space limit
   *
   *
   * degree of replication
   * replication status
   * max number of versions
   * space available
   *
   * An application to choose an implementation from the available
   * CabinetFactory implementations which has the desired capabilities.
   *
   * @return java.util.Map
   *
   */
  public java.util.Map getProperties() {
    return this.properties;
  }

  /**
   * Method getCabinet
   *
   *
   * @param guid
   *
   * @return Cabinet with given GUID
   *
   */
  public static Cabinet getCabinet(org.okiproject.serviceapi.guid.GUID guid) {
    throw new java.lang.UnsupportedOperationException();
  }

  /**
   * Method createCabinet
   *
   * create new empty cabinet in this factory
   *
   * @return Cabinet
   *
   */
  public Cabinet createCabinet() {
    throw new java.lang.UnsupportedOperationException();
  }
}

