package org.okiproject.serviceapi.logging;



/*
Copyright (c) 2001 Massachusetts Institute of Technology

Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice
shall be included in all copies or substantial portions of
the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
*/

/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/logging/LogSpecification.java,v $
 */

/**
Purpose: Represents the logging service abstraction. This will typically
be a singleton per JVM.
<p>
@version $Revision: 1.7 $ / $Date: 2002/03/07 22:23:24 $
*/
public interface LogSpecification
extends java.io.Serializable {

  /**
	*       Method addLog
	*
	*       Link a log to the collection. Each log serves as a kind of handler
	*       for logging.
	*
	*       @param who Represents the Agent state for authn/authz purposes.
	*       @param log   Represents the specific logging event handler.
	*
	* @throws LogSpecificationException if some error occurs while linking
	*       this handler.
	*/
  public void addLog(org.okiproject.serviceapi.common.Agent who,
                     final Log log)
  throws LogSpecificationException;

  /**
	*       Method removeLog
	*
	*       Unlinks a log from the collection.
	*
	*       @param who Represents the Agent state for authn/authz purposes.
	*       @param log   Represents the specific logging event handler.
	*
	*       @throws LogSpecificationException if some error occurs while
	*       unlinking this handler.
	*/
  public void removeLog(org.okiproject.serviceapi.common.Agent who,
                        final Log log)
  throws LogSpecificationException;

  /**
	 *
	 *       Method setBasePriority
	 *
   *       Sets the threshold priority for the logging service.
   *       Messages at a priority lower than this level will be ignored.
   *
   *       @param priority Represents the system logging priority.
	 */
  public void setBasePriority(final LogLevel priority);

  /**
	 *
	 *       Method getBasePriority
	 *
   *        Gets the threshold priority for the logging service.
	 *
  */
  public LogLevel getBasePriority();

}
