package org.okiproject.serviceapi.sched;
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/sched/SchedFactory.java,v $
 */
/**
<P>
<H2>ScheduleFactory</H2>
<P>
This factory abstracts the creational aspects of the API, enabling 
implementations to be loaded and used.
<p>
 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.1 $ / $Date: 2002/02/27 21:07:26 $
*/

public interface SchedFactory
extends org.okiproject.serviceapi.common.Factory {

  /**
   * Method createLocation
   *
   * @param locationName
   * 
   * @return Location
   *
   * @throws Exception
   *
   */
  public Location createLocation( final String locationName )
  throws Exception;


  /**
   * Method createLocation
   * Create an object that implements the Location interface.
   * 
   * @param locationName
   * @param locationType
   *
   * @return Location
   *
   * @throws Exception
   *
   */
  public Location createLocation( final String locationName, 
																	final String locationType ) throws Exception;

  /**
   * Method createTimeSpan
   *
   * @param start
   * @param end
   *
   * @return TimeSpan
   *
   * @throws Exception
   *
   */
  public TimeSpan createTimeSpan( final long start, final long end )
  throws Exception;

  /**
   * Method createTimeSpan
   *
   * @param start
   * @param end
   *
   * @return TimeSpan
   *
   * @throws Exception
   *
   */
  public TimeSpan createTimeSpan( java.util.Calendar start, final  java.util.Calendar end )
  throws Exception;

  /**
   * Method createScheduleItem

   * Creates a new ScheduleItem for the TimeSpan and the Resources specified,
   * optionally with a certain Location for this item.
   *
   * @param resources
   * @param timespan
   * @param location
   *
   * @return org.okiproject.serviceapi.sched.ScheduleItem
   *
   * @throws Exception
   *
   */
  public ScheduleItem createScheduleItem( 
								org.okiproject.serviceapi.common.Resource[] resources, 
								TimeSpan timespan, 
								Location location )
  throws Exception;

  /**
   * Method createResourceCommitment

   * Creates a new ResourceCommitment for the Resource and the ResourceStatus 
	 * specified.
   *
   * @param	resourceStatus
   * @param status
   *
   * @return org.okiproject.serviceapi.sched.ResourceStatus
   *
   * @throws Exception
   *
   */
  public ResourceCommitment createResourceCommitment( org.okiproject.serviceapi.common.Resource resource, ResourceStatus resourceStatus )
  throws Exception;

  /**
   * Method createScheduleManager
   * Creates a new ScheduleManager
   *
   * @return org.okiproject.serviceapi.sched.ScheduleManager
   *
   * @throws Exception
   *
   */
  public ScheduleManager createScheduleManager()
  throws Exception;

}
