
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/test/roles/roles_api_example.java,v $ 
 */
/**
<p>
*****************************************************************************
*
*  Sample pseudo-application demonstrating Roles API
*     11/25/2001
*
****************************************************************************

 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.3 $ / $Date: 2002/02/16 05:02:54 $
*/

public class roles_api_example {

  // Set some constants

  static final Boolean TRUE  = new Boolean(true);
  static final Boolean FALSE = new Boolean(false);

  /**
   * Method main
   *
   *
   * @param args
   *
   */
  public static void main(String[] args) {
    try {

      // Show the user the list of symbolic DB Names.

      String[] symbolicDBnameChoices =
        org.okiproject.serviceapi.roles.RolesFactory.getDatabaseSymbolicNames();

      System.out.println("Symbolic DB name choices:");

      for (int i = 0; symbolicDBnameChoices.length != i; i++) {
        System.out.println("     " + symbolicDBnameChoices[i]);
      }

      System.out.println("end of name choices");
      System.out.println();

      // Suppose the user chooses symbolic DB Name "prod_roles".

      // (Choices might be production, test, etc.)

      // Get the MIT implementation of the Roles interface (currently, your only

      // choice) and pass it a service name (e.g. "Roles") that this application

      // uses to identify this instance of roles and the symbolic DB Name.

      org.okiproject.serviceapi.roles.RolesFactory myFactory =
        org.okiproject.serviceapi.roles.RolesFactory.getDefaultRolesFactory(
          "Roles", "prod_roles");

      /*************************************************************************
      * Get a list of authorizations for a user.  Suppose his Kerberos username
      * is "repa" and we are interested in the category "GRAD" (Graduate
      * Admissions).
      *************************************************************************/
      org.okiproject.serviceapi.roles.Person           person1   =
        myFactory.newPerson("repa");
      org.okiproject.serviceapi.roles.FunctionCategory category1 =
        myFactory.newFunctionCategory("GRAD");
      org.okiproject.serviceapi.roles.Authorizations   authList1 =
        myFactory.getAuthsSimpleForm(person1,    // 1st argument is a person
                                                                     category1,    // 2nd argument is a function category
                                                                     TRUE,    // Let's only find auths. that are active today
                                                                     FALSE,    // Let's not expand Functions
                                                                     TRUE    // Let's expand Qualifier
                                                                       );

      System.out.println(person1.getKey()
                         + " has the following authorizations in category: "
                         + category1.getCategory());

      while (authList1.hasMoreAuthorizations()) {    // Print the list of auths.

        System.out.println(authList1.nextAuthorization());
      }

      System.out.println("end of authorizations for " + person1.getKey());
      System.out.println();

      /*************************************************************************
      ** Find out whether a given Person can do a given Function with a given
      **   Qualifier.  We are interested in Kerberos username "repa".
      **   The Function will be in category "GRAD" with
      **   functionName "ADM_ASSIGN_MIT_ID".  The Qualifier will be of
      **   qualifierType "AORG" and have qualifierCode "10"
      **   (Chemical Engineering).
      *************************************************************************/
      org.okiproject.serviceapi.roles.Person    person2    =
        myFactory.newPerson("repa");
      org.okiproject.serviceapi.roles.Function  function2  =
        myFactory.newFunction("GRAD", "ADM_ASSIGN_MIT_ID");
      org.okiproject.serviceapi.roles.Qualifier qualifier2 =
        myFactory.newQualifier("AORG", "10");

      try {
        Boolean heCanDoIt = myFactory.isAuthorized(person2, function2,
                                                   qualifier2);

        if (heCanDoIt.booleanValue()) {
          System.out.println(person2.getKey() + " can do Function "
                             + function2.getName() + " with Qualifier "
                             + qualifier2.getCode() + " "
                             + qualifier2.getQualifierType().getType());
          System.out.println();
        }
        else {
          System.out.println(person2.getKey()
                             + " is out of luck if he wants to do Function "
                             + function2.getName() + "\nwith Qualifier "
                             + qualifier2.getCode() + " "
                             + qualifier2.getQualifierType().getType());
          System.out.println();
        }
      }
      catch (Exception ex) {
        System.out.println(ex.getMessage());
      }

      /*************************************************************************
      ** Get a list of people who can do a given Function with a given Qualifier
      *************************************************************************/
      org.okiproject.serviceapi.roles.Function  function3  =
        myFactory.newFunction("SAP", "CAN SPEND OR COMMIT FUNDS");
      org.okiproject.serviceapi.roles.Qualifier qualifier3 =
        myFactory.newQualifier("FUND", "F1652700");

      org.okiproject.serviceapi.roles.People    people     =
        myFactory.getWhoCanDo(function3, qualifier3, TRUE);

      System.out.println("People who can do function " + function3.getName()
                         + " in category "
                         + function3.getFunctionCategory().getCategory()
                         + "\nwith qualifier " + qualifier3.getCode()
                         + " with type "
                         + qualifier3.getQualifierType().getType());

      if (null != people) {
        while (people.hasMorePeople()) {         // Print the list of people

          org.okiproject.serviceapi.roles.Person person = people.nextPerson();

          System.out.println("  person: " + person.getKey());
        }
      }

      System.out.println("end of list of people");
      System.out.println();

      /*************************************************************************
      ** Delete Active Authorizations
      ************************************************************************/
      org.okiproject.serviceapi.roles.Person         person4a    = null;
      org.okiproject.serviceapi.roles.Function       function4a  = null;
      org.okiproject.serviceapi.roles.Qualifier      qualifier4a = null;
      org.okiproject.serviceapi.roles.Authorizations authList4a  = null;

      try {
        person4a    = myFactory.newPerson("cshubert");
        function4a  = myFactory.newFunction("TEST", "TESTFUN1");
        qualifier4a = myFactory.newQualifier("ORGU", "001702");
        authList4a  = myFactory.getAuthsSimpleForm(person4a,    // 1st argument is a person
                                                   function4a,    // 2nd argument is a function
                                                   qualifier4a,    // 3rd argument is a qualifier
                                                   TRUE,    // Let's only find auths. that are active today
                                                   FALSE,    // Let's not expand Functions
                                                   TRUE    // Let's expand Qualifier
                                                     );

        System.out.println("Delete active authorizations");

        while (authList4a.hasMoreAuthorizations()) {
          org.okiproject.serviceapi.roles.Authorization auth4a =
            authList4a.nextAuthorization();

          System.out.println("Delete active authorization: " + auth4a);
          auth4a.delete();
        }

        System.out.println("end of delete active authorizations");
        System.out.println();
      }
      catch (Exception ex4a) {
        System.out.println("delete active authorization failure: "
                           + ex4a.getMessage());
      }

      /*************************************************************************
      ** Delete Inactive Authorizations
      ************************************************************************/
      org.okiproject.serviceapi.roles.Authorizations authList4aa = null;

      try {
        authList4aa = myFactory.getAuthsSimpleForm(person4a,    // 1st argument is a person
                                                   function4a,    // 2nd argument is a function
                                                   qualifier4a,    // 3rd argument is a qualifier
                                                   FALSE,    // Let's only find auths. that are not active today
                                                   FALSE,    // Let's not expand Functions
                                                   TRUE    // Let's expand Qualifier
                                                     );

        System.out.println("Delete inactive authorizations");

        if (authList4aa != null) {
          while (authList4aa.hasMoreAuthorizations()) {
            org.okiproject.serviceapi.roles.Authorization auth4aa =
              authList4aa.nextAuthorization();

            System.out.println("Delete inactive authorization: " + auth4aa);
            auth4aa.delete();
          }
        }

        System.out.println("end of delete inactive authorizations");
        System.out.println();
      }
      catch (Exception ex4aa) {
        System.out.println("delete inactive authorization failure: "
                           + ex4aa.getMessage());
      }

      /*************************************************************************
      ** Create a new Authorization
      ************************************************************************/
      org.okiproject.serviceapi.roles.Person         person4        =
        myFactory.newPerson("cshubert");
      org.okiproject.serviceapi.roles.Function       function4      =
        myFactory.newFunction("TEST", "TESTFUN1");
      org.okiproject.serviceapi.roles.Qualifier      qualifier4     =
        myFactory.newQualifier("ORGU", "001702");
      org.okiproject.serviceapi.roles.GrantPrivilege priv4          =
        myFactory.newGrantPrivilege(
          org.okiproject.serviceapi.roles.GrantPrivilege.NOTGRANTABLE);
      org.okiproject.serviceapi.roles.Authorization  authorization4 = null;

      try {
        authorization4 = myFactory.createAuthorization(person4, function4,
                qualifier4, priv4, TRUE);

        System.out.println(person4.getKey()
                           + " is now authorized to do Function "
                           + function4.getName() + "\nwith qualifier "
                           + qualifier3.getCode() + " with type "
                           + qualifier3.getQualifierType().getType()
                           + "\nstarting on "
                           + authorization4.getEffectiveDate());
        System.out.println();
      }
      catch (Exception ex4) {
        System.out.println("create authorization failure: "
                           + ex4.getMessage());
      }

      /*************************************************************************
      ** Update an Authorization
      ************************************************************************/
      priv4 = myFactory.newGrantPrivilege(
        org.okiproject.serviceapi.roles.GrantPrivilege.GRANTABLEFORGRANTING);

      java.util.GregorianCalendar effCal  =
        new java.util.GregorianCalendar(2001, java.util.Calendar.NOVEMBER, 1);
      long                        effTime = effCal.getTime().getTime();
      java.sql.Timestamp          effDate = new java.sql.Timestamp(effTime);
      java.util.GregorianCalendar expCal  =
        new java.util.GregorianCalendar(2001, java.util.Calendar.NOVEMBER, 2);
      long                        expTime = expCal.getTime().getTime();
      java.sql.Timestamp          expDate = new java.sql.Timestamp(expTime);

      try {
        authorization4.update(person4, function4, qualifier4, priv4, TRUE,
                              effDate, expDate);
        System.out.println("updated starting on "
                           + authorization4.getEffectiveDate() + " until "
                           + authorization4.getExpirationDate()
                           + "\nmodified: "
                           + authorization4.getModifiedDate());
        System.out.println();
      }
      catch (Exception ex4) {
        System.out.println("update authorization failure: "
                           + ex4.getMessage());
      }

      /*************************************************************************
       ** Get a list of the Functions available within a given category
       *************************************************************************/
      org.okiproject.serviceapi.roles.FunctionCategory category5 =
        myFactory.newFunctionCategory("WRHS");

      try {
        org.okiproject.serviceapi.roles.Functions functionList5 =
          myFactory.getFunctionsInCategory(category5);

        System.out.println("List of Functions");

        while (functionList5.hasMoreFunctions()) {    // Print list of functions

          org.okiproject.serviceapi.roles.Function func =
            functionList5.nextFunction();

          System.out.println("  Category: " + category5.getCategory()
                             + ", Function: " + func.getName());
        }

        System.out.println("end list of Functions");
        System.out.println();
      }
      catch (Exception ex5) {
        System.out.println(ex5.getMessage());
      }
    }
    catch (Exception exx) {
      System.out.println(exx.getMessage());
      exx.printStackTrace();
    }
  }
}
