package org.okiproject.serviceimpl.mit.authen ;
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceimpl/mit/authen/AuthenFactory.java,v $ 
 */
/**
<p>
The Authen API does not contain information that is expected
to be part of the implementation. There may be additional
implementation information that will need to be exposed in
this API.


 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.7 $ / $Date: 2002/02/16 17:10:12 $
*/

public class AuthenFactory
	extends org.okiproject.serviceapi.authentication.AuthenFactory
{
	private static final org.okiproject.serviceapi.authentication.AuthenException UNIMPLEMENTED
		= new org.okiproject.serviceapi.authentication.AuthenException
			( org.okiproject.serviceapi.authentication.AuthenException.UNIMPLEMENTED
			) ;

	private static final String KERBEROS4 = "KERBEROS 4" ;

	/**
		Returns an Agent that may or may not be anonymous
	*/
	public org.okiproject.serviceapi.common.Agent getAgent()
		throws Throwable
	{
		return getAgentInternal( null , KERBEROS4 ) ;
	}

	/**
		Returns the types of Authentication that are available
		to the current implementation
	*/
	public String[] getAuthenTypes()
		throws Exception
	{
		String[] types = new String[ 1 ] ;
		types[ 0 ] = KERBEROS4 ;
		return types ;
	}

	/**
		Returns the types of Authentication that this
		agent currently has.
	*/
	public String[] getAuthenTypes
		( org.okiproject.serviceapi.common.Agent agent
		)
		throws Exception
	{
		if( agent instanceof org.okiproject.serviceimpl.mit.common.Person )
		{
			org.okiproject.serviceimpl.mit.common.Person person
				= ( org.okiproject.serviceimpl.mit.common.Person ) agent ;
			String[] types
				= ( String[] ) person.getProperty
					( org.okiproject.serviceimpl.mit.common.Person.AUTHEN_TYPES ) ;
			return types ;
		}
		throw UNIMPLEMENTED ;
	}

	public org.okiproject.serviceapi.common.Agent getAgent
		( org.okiproject.serviceapi.common.Agent agent
		, String type
		)
		throws Throwable
	{
System.out.println( "getAgent" ) ;
System.out.println( agent ) ;
System.out.println( type ) ;
		if
			(  ( -1 != type.toLowerCase().indexOf( "kerberos" ) )
			&& ( agent instanceof org.okiproject.serviceimpl.mit.common.Person )
			)
		{
			return getAgentInternal( agent , KERBEROS4 ) ;
		}
		throw UNIMPLEMENTED ;
	}

	public void destroyAuthen
		( org.okiproject.serviceapi.common.Agent agent
		)
		throws Exception
	{
		if
			(  ( agent instanceof org.okiproject.serviceimpl.mit.common.Person )
			)
		{
			org.okiproject.serviceimpl.mit.common.Person person
				= ( org.okiproject.serviceimpl.mit.common.Person ) agent ;
			person.setProperty
				( org.okiproject.serviceimpl.mit.common.Person.AUTHEN_TYPES
				, new String[ 0 ]
				) ;
			return ;
		}
		throw UNIMPLEMENTED ;
	}

	public void destroyAuthen
		( org.okiproject.serviceapi.common.Agent agent
		, String type
		)
		throws Exception
	{
		if
			(  ( -1 != type.toLowerCase().indexOf( "kerberos" ) )
			&& ( agent instanceof org.okiproject.serviceimpl.mit.common.Person )
			)
		{
			org.okiproject.serviceimpl.mit.common.Person person
				= ( org.okiproject.serviceimpl.mit.common.Person ) agent ;
			person.setProperty
				( org.okiproject.serviceimpl.mit.common.Person.AUTHEN_TYPES
				, new String[ 0 ]
				) ;
			return ;
		}
		throw UNIMPLEMENTED ;
	}

	public void destroyAuthen
		( org.okiproject.serviceapi.common.Agent agent
		, String[] types
		)
		throws Exception
	{
		if
			(  isIn( types , "kerberos" )
			&& ( agent instanceof org.okiproject.serviceimpl.mit.common.Person )
			)
		{
			org.okiproject.serviceimpl.mit.common.Person person
				= ( org.okiproject.serviceimpl.mit.common.Person ) agent ;
			person.setProperty
				( org.okiproject.serviceimpl.mit.common.Person.AUTHEN_TYPES
				, new String[ 0 ]
				) ;
			return ;
		}
		throw UNIMPLEMENTED ;
	}

	private boolean isIn( String[] types , String type )
	{
		if( ( null != types ) && ( null != type ) )
		{
			for( int i = 0 ; types.length != i ; i ++ )
			{
				if( -1 != types[ i ].toLowerCase().indexOf( type ) )
				{
					return true ;
				}
			}
		}
		return false ;
	}

	private org.okiproject.serviceapi.common.Agent getAgentInternal
		( org.okiproject.serviceapi.common.Agent agent
		, String type
		)
		throws Throwable
	{
		javax.swing.JTextField name = new javax.swing.JTextField() ;
		name.setColumns( 10 ) ;
		javax.swing.JLabel nameLabel = new javax.swing.JLabel( "Name" ) ;
		nameLabel.setLabelFor( name ) ;
		javax.swing.JPasswordField password = new javax.swing.JPasswordField() ;
		password.setColumns( 10 ) ;
		javax.swing.JLabel passwordLabel = new javax.swing.JLabel( "Password" ) ;
		passwordLabel.setLabelFor( password ) ;
		javax.swing.JLabel quitLabel = new javax.swing.JLabel( "To finish close dialog" ) ;
		javax.swing.JFrame parent = new javax.swing.JFrame( "Kerberos Authentication" ) ;
		javax.swing.JDialog dialog = new javax.swing.JDialog( parent ) ;
		dialog.setModal( true ) ;
		dialog.getContentPane().setLayout( new java.awt.FlowLayout() ) ;
		dialog.getContentPane().setSize( 100 , 100 ) ;
		dialog.getContentPane().add( nameLabel ) ;
		dialog.getContentPane().add( name ) ;
		dialog.getContentPane().add( passwordLabel ) ;
		dialog.getContentPane().add( password ) ;
		dialog.getContentPane().add( quitLabel ) ;
		dialog.pack();
		dialog.show() ;
		Object ticket
			= org.okiproject.serviceimpl.mit.krb4.Authenticate.getUserCredentials
				( name.getText()
				, ""
				, password.getPassword()
				) ;
		String[] types = new String[ 1 ] ;
		types[ 0 ] = KERBEROS4 ;
		if( null == agent )
		{
			java.util.Properties props = new java.util.Properties() ;
			props.put
				( org.okiproject.serviceimpl.mit.common.Person.KERBEROS_NAME
				, name.getText()
				) ;
			props.put
				( org.okiproject.serviceimpl.mit.common.Person.CREDENTIALS
				, ticket
				) ;
			props.put
				( org.okiproject.serviceimpl.mit.common.Person.AUTHEN_TYPES
				, types
				) ;
			agent = new org.okiproject.serviceimpl.mit.common.Person( props ) ;
		}
		else
		{
			( ( org.okiproject.serviceimpl.mit.common.Person ) agent ).setProperty
				( org.okiproject.serviceimpl.mit.common.Person.KERBEROS_NAME
				, name.getText()
				) ;
			( ( org.okiproject.serviceimpl.mit.common.Person ) agent ).setProperty
				( org.okiproject.serviceimpl.mit.common.Person.CREDENTIALS
				, ticket
				) ;
			( ( org.okiproject.serviceimpl.mit.common.Person ) agent ).setProperty
				( org.okiproject.serviceimpl.mit.common.Person.AUTHEN_TYPES
				, types
				) ;
		}
		return 	agent  ;
	}

}