package edu.mit.is.service.authentication.impl.authenkrb4 ;
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceimpl/mit/authen/Factory.java,v $
 */
/**
 * <p>
   The Authen API does not contain information that is expected
   to be part of the implementation. There may be additional
   implementation information that will need to be exposed in
   this API.
 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.1 $ / $Date: 2002/03/06 22:01:24 $
*/

public class Factory
	extends org.okip.service.authentication.api.Factory
{
	private static final org.okip.service.authentication.api.AuthenException UNIMPLEMENTED
		= new org.okip.service.authentication.api.Exception
			( org.okip.service.authentication.api.Exception.UNIMPLEMENTED
			) ;

	private static final String KERBEROS4 = "KERBEROS 4" ;

	/**
		Returns an Agent that may or may not be anonymous
	*/
	public org.okip.service.shared.api.Agent getAgent()
		throws org.okip.service.authentication.api.Exception
	{
		return getAgentInternal( null , KERBEROS4 ) ;
	}

	/**
		Returns the types of Authentication that are available
		to the current implementation
	*/
	public java.io.Serializable[] getAuthenTypes()
		throws org.okip.service.authentication.api.Exception
	{
		java.io.Serializable[] types = new String[ 1 ] ;
		types[ 0 ] = KERBEROS4 ;
		return types ;
	}

	/**
		Returns the types of Authentication that this
		agent currently has.
	*/
	public java.io.Serializable[] getAuthenTypes
		( org.okip.service.shared.api.Agent agent
		)
		throws org.okip.service.authentication.api.Exception
	{
		if( agent instanceof edu.mit.is.authentication.impl.authenkrb4.Person )
		{
			edu.mit.is.authentication.impl.authenkrb4.Person person
				= ( edu.mit.is.authentication.impl.authenkrb4.Person ) agent ;
			java.io.Serializable[] types
				= ( String[] ) person.getProperty
					( edu.mit.is.authentication.impl.authenkrb4.Person.AUTHEN_TYPES ) ;
			return types ;
		}
		throw org.okip.service.shared.api.Exception.UNIMPLEMENTED ;
	}

	public org.okip.service.shared.api.Agent getAgent
		( org.okip.service.shared.api.Agent agent
		, String type
		)
		throws org.okip.service.authentication.api.Exception
	{
		if
			(  ( -1 != type.toLowerCase().indexOf( "kerberos" ) )
			&& ( agent instanceof edu.mit.is.authentication.impl.authenkrb4.Person )
			)
		{
			return getAgentInternal( agent , KERBEROS4 ) ;
		}
		throw org.okip.service.shared.api.Exception.UNIMPLEMENTED ;
	}

	public void destroyAuthen
		( org.okip.service.shared.api.Agent agent
		)
		throws org.okip.service.authentication.api.Exception
	{
		if
			(  ( agent instanceof edu.mit.is.authentication.impl.authenkrb4.Person )
			)
		{
			edu.mit.is.authentication.impl.authenkrb4.Person person
				= ( edu.mit.is.authentication.impl.authenkrb4.Person ) agent ;
			person.setProperty
				( edu.mit.is.authentication.impl.authenkrb4.Person.AUTHEN_TYPES
				, new String[ 0 ]
				) ;
			return ;
		}
		throw org.okip.service.shared.api.Exception.UNIMPLEMENTED ;
	}

	public void destroyAuthen
		( org.okip.service.shared.api.Agent agent
		, String type
		)
		throws org.okip.service.authentication.api.Exception
	{
		if
			(  ( -1 != type.toLowerCase().indexOf( "kerberos" ) )
			&& ( agent instanceof edu.mit.is.authentication.impl.authenkrb4.Person )
			)
		{
			edu.mit.is.authentication.impl.authenkrb4.Person person
				= ( edu.mit.is.authentication.impl.authenkrb4.Person ) agent ;
			person.setProperty
				( edu.mit.is.authentication.impl.authenkrb4.Person.AUTHEN_TYPES
				, new String[ 0 ]
				) ;
			return ;
		}
		throw org.okip.service.shared.api.Exception.UNIMPLEMENTED ;
	}

	public void destroyAuthen
		( org.okip.service.shared.api.Agent agent
		, String[] types
		)
		throws org.okip.service.authentication.api.Exception
	{
		if
			(  isIn( types , "kerberos" )
			&& ( agent instanceof edu.mit.is.authentication.impl.authenkrb4.Person )
			)
		{
			edu.mit.is.authentication.impl.authenkrb4.Person person
				= ( edu.mit.is.authentication.impl.authenkrb4.Person ) agent ;
			person.setProperty
				( edu.mit.is.authentication.impl.authenkrb4.PersonAUTHEN_TYPES
				, new String[ 0 ]
				) ;
			return ;
		}
		throw org.okip.service.shared.api.Exception.UNIMPLEMENTED ;
	}

	private boolean isIn( String[] types , String type )
	{
		if( ( null != types ) && ( null != type ) )
		{
			for( int i = 0 ; types.length != i ; i ++ )
			{
				if( -1 != types[ i ].toLowerCase().indexOf( type ) )
				{
					return true ;
				}
			}
		}
		return false ;
	}

	private org.okip.service.shared.api.Agent getAgentInternal
		( org.okip.service.shared.api.Agent agent
		, String type
		)
		throws org.okip.service.authentication.api.Exception
	{
		javax.swing.JTextField name = new javax.swing.JTextField() ;
		name.setColumns( 10 ) ;
		javax.swing.JLabel nameLabel = new javax.swing.JLabel( "Name" ) ;
		nameLabel.setLabelFor( name ) ;
		javax.swing.JPasswordField password = new javax.swing.JPasswordField() ;
		password.setColumns( 10 ) ;
		javax.swing.JLabel passwordLabel = new javax.swing.JLabel( "Password" ) ;
		passwordLabel.setLabelFor( password ) ;
		javax.swing.JLabel quitLabel = new javax.swing.JLabel( "To finish close dialog" ) ;
		javax.swing.JFrame parent = new javax.swing.JFrame( "Kerberos Authentication" ) ;
		javax.swing.JDialog dialog = new javax.swing.JDialog( parent ) ;
		dialog.setModal( true ) ;
		dialog.getContentPane().setLayout( new java.awt.FlowLayout() ) ;
		dialog.getContentPane().setSize( 100 , 100 ) ;
		dialog.getContentPane().add( nameLabel ) ;
		dialog.getContentPane().add( name ) ;
		dialog.getContentPane().add( passwordLabel ) ;
		dialog.getContentPane().add( password ) ;
		dialog.getContentPane().add( quitLabel ) ;
		dialog.pack();
		dialog.show() ;
		Object ticket
			= org.okiproject.serviceimpl.mit.krb4.Authenticate.getUserCredentials
				( name.getText()
				, ""
				, password.getPassword()
				) ;
		String[] types = new String[ 1 ] ;
		types[ 0 ] = KERBEROS4 ;
		if( null == agent )
		{
			java.util.Properties props = new java.util.Properties() ;
			props.put
				( edu.mit.is.authentication.impl.authenkrb4.PersonKERBEROS_NAME
				, name.getText()
				) ;
			props.put
				( edu.mit.is.authentication.impl.authenkrb4.PersonCREDENTIALS
				, ticket
				) ;
			props.put
				( edu.mit.is.authentication.impl.authenkrb4.PersonAUTHEN_TYPES
				, types
				) ;
			agent = new edu.mit.is.authentication.impl.authenkrb4.Person( props ) ;
		}
		else
		{
			( ( edu.mit.is.authentication.impl.authenkrb4.Person ) agent ).setProperty
				( edu.mit.is.authentication.impl.authenkrb4.PersonKERBEROS_NAME
				, name.getText()
				) ;
			( ( edu.mit.is.authentication.impl.authenkrb4.Person ) agent ).setProperty
				( edu.mit.is.authentication.impl.authenkrb4.PersonCREDENTIALS
				, ticket
				) ;
			( ( edu.mit.is.authentication.impl.authenkrb4.Person ) agent ).setProperty
				( edu.mit.is.authentication.impl.authenkrb4.PersonAUTHEN_TYPES
				, types
				) ;
		}
		return 	agent  ;
	}

}