package org.okiproject.serviceimpl.mit.sql ;
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceimpl/mit/sql/Blob.java,v $ 
 */
/**
<p>
 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.3 $ / $Date: 2002/02/16 17:10:52 $
*/
public class Blob
	implements org.okiproject.serviceapi.dbc.Blob
{
	private static final String GETBYTES = "getBytes" ;
	private static final String POSITION = "position" ;
	private static final Long INVALIDPOSITION = new Long( -1 ) ;

	private static final org.okiproject.serviceapi.dbc.TableException LOCALBLOBGETBYTESFAILED
		= new org.okiproject.serviceapi.dbc.TableException
			( org.okiproject.serviceapi.dbc.TableException.BLOBGETBYTESFAILED
			) ;

	private static final org.okiproject.serviceapi.dbc.TableException LOCALBLOBPOSITIONFAILED
		= new org.okiproject.serviceapi.dbc.TableException
			( org.okiproject.serviceapi.dbc.TableException.BLOBPOSITIONFAILED
			) ;

	private org.okiproject.serviceimpl.mit.sql.Rows rows = null ;
	private int row = 0 ;
	private int col = 0 ;
	private Long pos = INVALIDPOSITION ;
	private Integer len = new Integer( 0 ) ;
	private byte[] pattern = null ;
	private Long start = INVALIDPOSITION ;

	private long length = -1 ;
	public long length() { return this.length ; }

	protected Blob
		( org.okiproject.serviceimpl.mit.sql.Rows rows
		, long length
		, int row
		, int col
		)
	{
		this.rows = rows ;
		this.row = row ;
		this.col = col ;
		this.length = length ;
	}

	public byte[] getBytes
		( Long pos
		, Integer len
		)
		throws Exception
	{
		this.pos = pos ;
		this.len = len ;
		org.okiproject.serviceimpl.mit.sql.Connection connection
			= ( org.okiproject.serviceimpl.mit.sql.Connection )
				rows.table.getConnection() ;
		byte[] bytes = null ;
		if( connection.hasClient() )
		{
			bytes = getBytesRemotely() ;
		}
		else
		{
			bytes = getBytesLocally() ;
		}
		return bytes ;
	}

	private byte[] getBytesRemotely()
		throws Exception
	{
		Object[] args = new Object[ 2 ] ;
		args[ 0 ] = this.pos ;
		args[ 1 ] = this.len ;
		Class[] argstypes = new Class[ 2 ] ;
		argstypes[ 0 ] = args[ 0 ].getClass() ;
		argstypes[ 1 ] = args[ 1 ].getClass() ;
		org.okiproject.serviceimpl.mit.sql.Connection connection
			= ( org.okiproject.serviceimpl.mit.sql.Connection )
				rows.table.getConnection() ;
		org.okiproject.serviceimpl.mit.rmissl.Client client = connection.getClient() ;
		return ( byte[] ) client.invoke( this , GETBYTES , argstypes , args ) ;
	}

	public byte[] getBytesLocally()
		throws Exception
	{
		java.sql.ResultSet rs = null ;
		try
		{
			rs = rows.getResultSet() ;
			int rsType = rs.getType() ;
			if( rs.TYPE_FORWARD_ONLY == rsType )
			{
				if( rs.next() )
				{
					return getBytesFromResultSet( rs ) ;
				}
			}
			else if( rs.TYPE_SCROLL_INSENSITIVE == rsType )
			{
				rs.setFetchDirection( rs.FETCH_FORWARD ) ;
				if( rs.first() )
				{
					return getBytesFromResultSet( rs ) ;
				}
			}
			else if( rs.TYPE_SCROLL_SENSITIVE == rsType )
			{
				rs.setFetchDirection( rs.FETCH_FORWARD ) ;
				boolean isRowAvailable = rs.first() ;
				if( rs.first() )
				{
					return getBytesFromResultSet( rs ) ;
				}
			}
			java.sql.Statement statement = rs.getStatement() ;
			rs.close() ;
			statement.close() ;
		}
		catch( Exception ex )
		{
			if( null != rs )
			{
				rs.close() ;
			}
			String message = LOCALBLOBGETBYTESFAILED.getMessage() ;
			message += ex.getMessage() ;
			throw new org.okiproject.serviceapi.dbc.TableException( message ) ;
		}
		return null ;
	}

	private byte[] getBytesFromResultSet
		( java.sql.ResultSet rs
		)
		throws Exception
	{
		int cursor = 0 ;
		boolean isRowAvailable = true ;
		while( isRowAvailable )
		{
			if( cursor == this.row )
			{
				java.sql.Blob blob = ( java.sql.Blob ) rs.getObject( this.col + 1 ) ;
				return blob.getBytes( this.pos.longValue() , this.len.intValue() ) ;
			}
			cursor ++ ;
			isRowAvailable = rs.next() ;
		}
		return null ;
	}

	public Long position
		( byte[] pattern
		, Long start
		)
		throws Exception
	{
		this.pattern = pattern ;
		this.start = start ;

		org.okiproject.serviceimpl.mit.sql.Connection connection
			= ( org.okiproject.serviceimpl.mit.sql.Connection )
				rows.table.getConnection() ;
		Long thePosition = INVALIDPOSITION ;
		if( connection.hasClient() )
		{
			thePosition = positionRemotely() ;
		}
		else
		{
			thePosition = positionLocally() ;
		}
		return thePosition ;
	}

	private Long positionRemotely()
		throws Exception
	{
		Object[] args = new Object[ 2 ] ;
		args[ 0 ] = this.pattern ;
		args[ 1 ] = this.start ;
		Class[] argstypes = new Class[ 2 ] ;
		argstypes[ 0 ] = args[ 0 ].getClass() ;
		argstypes[ 1 ] = args[ 1 ].getClass() ;
		org.okiproject.serviceimpl.mit.sql.Connection connection
			= ( org.okiproject.serviceimpl.mit.sql.Connection )
				rows.table.getConnection() ;
		org.okiproject.serviceimpl.mit.rmissl.Client client = connection.getClient() ;
		return ( Long ) client.invoke( this , POSITION , argstypes , args ) ;
	}

	private Long positionLocally()
		throws Exception
	{
		java.sql.ResultSet rs = null ;
		try
		{
			rs = rows.getResultSet() ;
			int rsType = rs.getType() ;
			if( rs.TYPE_FORWARD_ONLY == rsType )
			{
				if( rs.next() )
				{
					return positionFromResultSet( rs ) ;
				}
			}
			else if( rs.TYPE_SCROLL_INSENSITIVE == rsType )
			{
				rs.setFetchDirection( rs.FETCH_FORWARD ) ;
				if( rs.first() )
				{
					return positionFromResultSet( rs ) ;
				}
			}
			else if( rs.TYPE_SCROLL_SENSITIVE == rsType )
			{
				rs.setFetchDirection( rs.FETCH_FORWARD ) ;
				boolean isRowAvailable = rs.first() ;
				if( rs.first() )
				{
					return positionFromResultSet( rs ) ;
				}
			}
			java.sql.Statement statement = rs.getStatement() ;
			rs.close() ;
			statement.close() ;
		}
		catch( Exception ex )
		{
			if( null != rs )
			{
				rs.close() ;
			}
			String message = LOCALBLOBPOSITIONFAILED.getMessage() ;
			message += ex.getMessage() ;
			throw new org.okiproject.serviceapi.dbc.TableException( message ) ;
		}
		return INVALIDPOSITION ;
	}

	private Long positionFromResultSet
		( java.sql.ResultSet rs
		)
		throws Exception
	{
		int cursor = 0 ;
		boolean isRowAvailable = true ;
		while( isRowAvailable )
		{
			if( cursor == this.row )
			{
				java.sql.Blob blob = ( java.sql.Blob ) rs.getObject( this.col + 1 ) ;
				return new Long( blob.position( this.pattern , this.start.longValue() ) ) ;
			}
			cursor ++ ;
			isRowAvailable = rs.next() ;
		}
		return INVALIDPOSITION ;
	}

}
